-- Schema for tracking DebConf video recordings in Pentabarf's database
-- TODO:
-- - represent post-processing filters and associate them with event_recording
-- - add names of reviewers
-- - add some way to flag events that are contained in other events, so they
--   don't get their own target files

-- Status (quality) of a video file.
CREATE TABLE video_file_status (
    id serial PRIMARY KEY,
    -- X for unknown, otherwise probably A, B, C or F.
    file_status_code character NOT NULL UNIQUE,
    -- Description of what the status means.
    file_status_desc character varying(1000) NOT NULL
);

-- Recorded file.
CREATE TABLE video_recording (
    id serial PRIMARY KEY,
    room_id integer NOT NULL REFERENCES rooms,
    recording_time timestamp NOT NULL,
    recording_duration interval NOT NULL,
    recording_filename character varying(100) NOT NULL UNIQUE,
    file_status_id integer NOT NULL REFERENCES video_file_status
);

-- Association of events with their recordings.  This is separate
-- from recordings because events are not always be recorded in a
-- single file.
CREATE TABLE video_event_recording (
    id serial PRIMARY KEY,
    event_id integer NOT NULL REFERENCES events,
    recording_id integer NOT NULL REFERENCES video_recording,
    -- Start and end of the event recording within the file.
    start_time interval(3) NOT NULL,
    end_time interval(3) NOT NULL,
    event_recording_base_name character varying(150) NOT NULL,
    UNIQUE (event_id, recording_id),
    CHECK (start_time < end_time)
);

-- Target format - covers both archive and final formats.
-- All rates are in bit/s or Hz.  If video_width, video_height,
-- audio_sample_rate or audio_channel_count is 0 this means the
-- corresponding property is not changed from the recording.
-- Format and codec names must be understood by libav{format,codec}.
CREATE TABLE video_target_format (
    id serial PRIMARY KEY,
    target_format_abbr character varying(20) NOT NULL UNIQUE,
    -- Priority of the target format, for scheduling purposes.  Higher
    -- number -> higher priority.
    priority integer NOT NULL,
    container_name character varying(10) NOT NULL,
    filename_extension character varying(10) NOT NULL,
    video_width integer NOT NULL CHECK (video_width >= 0),
    video_height integer NOT NULL CHECK (video_height >= 0),
    video_codec_name character varying(20) NOT NULL,
    video_bit_rate integer NOT NULL CHECK (video_bit_rate >= 10000),
    audio_sample_rate integer NOT NULL
    CHECK (audio_sample_rate = 0 OR audio_sample_rate >= 10000),
    audio_channel_count integer NOT NULL
    CHECK (audio_channel_count >= 0 AND audio_channel_count <= 2),
    audio_codec_name character varying(20) NOT NULL,
    audio_bit_rate integer NOT NULL CHECK (audio_bit_rate >= 10000)
);

-- Target file that has been created.  Its name can be inferred
-- from other relations.
CREATE TABLE video_target_file (
    id serial PRIMARY KEY,
    event_recording_id integer NOT NULL REFERENCES video_event_recording,
    target_format_id integer NOT NULL REFERENCES video_target_format,
    file_status_id integer NOT NULL REFERENCES video_file_status,
    UNIQUE (event_recording_id, target_format_id)
);

-- Subject of events.  This is used for categorising events on DVDs
-- and maybe for normal web pages.
-- CREATE TABLE subject (
--     subject_id serial PRIMARY KEY,
--     subject_name character varying(40) NOT NULL UNIQUE
-- );

-- Associaton of events and subjects.
-- CREATE TABLE event_subject (
--     event_id integer NOT NULL REFERENCES event,
--     subject_id integer NOT NULL REFERENCES subject,
--     UNIQUE (event_id, subject_id)
-- );
